@@ -154,12 +154,18 @@ vty_log_out (struct vty *vty, const char *level, const char *proto_str,
   int ret;
   int len;
   char buf[1024];
+  struct tm *tm;
+
+  if ((tm = localtime(&recent_time.tv_sec)) != NULL)
+    len = strftime(buf, sizeof(buf), "%Y/%m/%d %H:%M:%S ", tm);
+  else
+    len = 0;
 
   if (level)
-    len = snprintf(buf, sizeof(buf), "%s: %s: ", level, proto_str);
+    ret = snprintf(buf+len, sizeof(buf)-len, "%s: %s: ", level, proto_str);
   else
-    len = snprintf(buf, sizeof(buf), "%s: ", proto_str);
-  if ((len < 0) || ((size_t)len >= sizeof(buf)))
+    ret = snprintf(buf+len, sizeof(buf)-len, "%s: ", proto_str);
+  if ((ret < 0) || ((size_t)(len += ret) >= sizeof(buf)))
     return -1;
 
   if (((ret = vsnprintf(buf+len, sizeof(buf)-len, format, va)) < 0) ||
@@ -176,10 +182,14 @@ vty_log_out (struct vty *vty, const char *level, const char *proto_str,
 	   drop the data and ignore. */
 	return -1;
       /* Fatal I/O error. */
+      vty->monitor = 0; /* disable monitoring to avoid infinite recursion */
       zlog_warn("%s: write failed to vty client fd %d, closing: %s",
 		__func__, vty->fd, safe_strerror(errno));
       buffer_reset(vty->obuf);
-      vty_close(vty);
+      /* cannot call vty_close, because a parent routine may still try
+         to access the vty struct */
+      vty->status = VTY_CLOSE;
+      shutdown(vty->fd, SHUT_RDWR);
       return -1;
     }
   return 0;
@@ -228,7 +238,7 @@ vty_hello (struct vty *vty)
 	    {
 	      char *s;
 	      /* work backwards to ignore trailling isspace() */
-	      for (s = buf + strlen (buf); (s > buf) && isspace (*(s - 1));
+	      for (s = buf + strlen (buf); (s > buf) && isspace ((int)*(s - 1));
 		   s--);
 	      *s = '\0';
 	      vty_out (vty, "%s%s", buf, VTY_NEWLINE);
@@ -985,18 +995,12 @@ vty_describe_command (struct vty *vty)
   switch (ret)
     {
     case CMD_ERR_AMBIGUOUS:
-      cmd_free_strvec (vline);
       vty_out (vty, "%% Ambiguous command.%s", VTY_NEWLINE);
-      vty_prompt (vty);
-      vty_redraw_line (vty);
-      return;
+      goto out;
       break;
     case CMD_ERR_NO_MATCH:
-      cmd_free_strvec (vline);
       vty_out (vty, "%% There is no matched command.%s", VTY_NEWLINE);
-      vty_prompt (vty);
-      vty_redraw_line (vty);
-      return;
+      goto out;
       break;
     }  
 
@@ -1066,8 +1070,10 @@ vty_describe_command (struct vty *vty)
 	vty_describe_fold (vty, width, desc_width, desc);
     }
 
+out:
   cmd_free_strvec (vline);
-  vector_free (describe);
+  if (describe)
+    vector_free (describe);
 
   vty_prompt (vty);
   vty_redraw_line (vty);
@@ -1347,6 +1353,7 @@ vty_read (struct thread *thread)
 	      vty_event (VTY_READ, vty_sock, vty);
 	      return 0;
 	    }
+	  vty->monitor = 0; /* disable monitoring to avoid infinite recursion */
 	  zlog_warn("%s: read error on vty client fd %d, closing: %s",
 		    __func__, vty->fd, safe_strerror(errno));
 	}
@@ -1569,6 +1576,7 @@ vty_flush (struct thread *thread)
   switch (flushrc)
     {
     case BUFFER_ERROR:
+      vty->monitor = 0; /* disable monitoring to avoid infinite recursion */
       zlog_warn("buffer_flush failed on vty client fd %d, closing",
 		vty->fd);
       buffer_reset(vty->obuf);
@@ -2016,6 +2024,7 @@ vtysh_flush(struct vty *vty)
       vty_event(VTYSH_WRITE, vty->fd, vty);
       break;
     case BUFFER_ERROR:
+      vty->monitor = 0; /* disable monitoring to avoid infinite recursion */
       zlog_warn("%s: write error to fd %d, closing", __func__, vty->fd);
       buffer_reset(vty->obuf);
       vty_close(vty);
@@ -2051,6 +2060,7 @@ vtysh_read (struct thread *thread)
 	      vty_event (VTYSH_READ, sock, vty);
 	      return 0;
 	    }
+	  vty->monitor = 0; /* disable monitoring to avoid infinite recursion */
 	  zlog_warn("%s: read failed on vtysh client fd %d, closing: %s",
 		    __func__, sock, safe_strerror(errno));
 	}
@@ -2134,7 +2144,10 @@ vty_serv_sock (const char *addr, unsigned short port, const char *path)
 #endif /* VTYSH */
 }
 
-/* Close vty interface. */
+/* Close vty interface.  Warning: call this only from functions that
+   will be careful not to access the vty afterwards (since it has
+   now been freed).  This is safest from top-level functions (called
+   directly by the thread dispatcher). */
 void
 vty_close (struct vty *vty)
 {
@@ -2408,6 +2421,9 @@ vty_log (const char *level, const char *proto_str,
 {
   unsigned int i;
   struct vty *vty;
+  
+  if (!vtyvec)
+    return;
 
   for (i = 0; i < vector_active (vtyvec); i++)
     if ((vty = vector_slot (vtyvec, i)) != NULL)
@@ -2427,6 +2443,10 @@ vty_log_fixed (const char *buf, size_t len)
   unsigned int i;
   struct iovec iov[2];
 
+  /* vty may not have been initialised */
+  if (!vtyvec)
+    return;
+  
   iov[0].iov_base = (void *)buf;
   iov[0].iov_len = len;
   iov[1].iov_base = (void *)"\r\n";
@@ -2740,6 +2760,13 @@ DEFUN (terminal_no_monitor,
   return CMD_SUCCESS;
 }
 
+ALIAS (terminal_no_monitor,
+       no_terminal_monitor_cmd,
+       "no terminal monitor",
+       NO_STR
+       "Set terminal line parameters\n"
+       "Copy debug output to the current terminal line\n")
+
 DEFUN (show_history,
        show_history_cmd,
        "show history",
@@ -2908,6 +2935,7 @@ vty_init (struct thread_master *master_thread)
   install_element (CONFIG_NODE, &show_history_cmd);
   install_element (ENABLE_NODE, &terminal_monitor_cmd);
   install_element (ENABLE_NODE, &terminal_no_monitor_cmd);
+  install_element (ENABLE_NODE, &no_terminal_monitor_cmd);
   install_element (ENABLE_NODE, &show_history_cmd);
 
   install_default (VTY_NODE);
