@@ -82,6 +82,9 @@ typedef int socklen_t;
 #ifdef HAVE_LIMITS_H
 #include <limits.h>
 #endif /* HAVE_LIMITS_H */
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif /* HAVE_INTTYPES_H */
 
 /* machine dependent includes */
 #ifdef SUNOS_5
@@ -120,6 +123,10 @@ typedef int socklen_t;
 #include <sys/prctl.h>
 #endif /* HAVE_LCAPS */
 
+#ifdef HAVE_SOLARIS_CAPABILITIES
+#include <priv.h>
+#endif /* HAVE_SOLARIS_CAPABILITIES */
+
 /* network include group */
 
 #include <sys/socket.h>
@@ -199,10 +206,27 @@ typedef int socklen_t;
 #include <netinet6/nd6.h>
 #endif /* HAVE_NETINET6_ND6_H */
 
-/* Some systems do not define UINT32_MAX */
+/* Some systems do not define UINT32_MAX, etc.. from inttypes.h
+ * e.g. this makes life easier for FBSD 4.11 users.
+ */
+#ifndef INT8_MAX
+#define INT8_MAX	(127)
+#endif
+#ifndef INT16_MAX
+#define INT16_MAX	(32767)
+#endif
+#ifndef INT32_MAX
+#define INT32_MAX	(2147483647)
+#endif
+#ifndef UINT8_MAX
+#define UINT8_MAX	(255U)
+#endif
+#ifndef UINT16_MAX
+#define UINT16_MAX	(65535U)
+#endif
 #ifndef UINT32_MAX
-#define UINT32_MAX 0xFFFFFFFFU
-#endif /* UINT32_MAX */
+#define UINT32_MAX	(4294967295U)
+#endif
 
 #ifdef HAVE_LIBUTIL_H
 #include <libutil.h>
@@ -332,6 +356,23 @@ struct in_pktinfo
 #define HAVE_IP_HDRINCL_BSD_ORDER
 #endif
 
+/* Define BYTE_ORDER, if not defined. Useful for compiler conditional
+ * code, rather than preprocessor conditional.
+ * Not all the world has this BSD define.
+ */
+#ifndef BYTE_ORDER
+#define BIG_ENDIAN	4321	/* least-significant byte first (vax, pc) */
+#define LITTLE_ENDIAN	1234	/* most-significant byte first (IBM, net) */
+#define PDP_ENDIAN	3412	/* LSB first in word, MSW first in long (pdp) */
+
+#if defined(WORDS_BIGENDIAN)
+#define BYTE_ORDER	BIG_ENDIAN
+#else /* !WORDS_BIGENDIAN */
+#define BYTE_ORDER	LITTLE_ENDIAN
+#endif /* WORDS_BIGENDIAN */
+
+#endif /* ndef BYTE_ORDER */
+
 /* MAX / MIN are not commonly defined, but useful */
 #ifndef MAX
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
@@ -345,6 +386,9 @@ struct in_pktinfo
 #define IN6_ARE_ADDR_EQUAL IN6_IS_ADDR_EQUAL
 #endif /* IN6_ARE_ADDR_EQUAL */
 
+/* default zebra TCP port for zclient */
+#define ZEBRA_PORT			2600
+
 /* Zebra message types. */
 #define ZEBRA_INTERFACE_ADD                1
 #define ZEBRA_INTERFACE_DELETE             2
@@ -370,6 +414,12 @@ struct in_pktinfo
 #define ZEBRA_ROUTER_ID_UPDATE            22
 #define ZEBRA_MESSAGE_MAX                 23
 
+/* Marker value used in new Zserv, in the byte location corresponding
+ * the command value in the old zserv header. To allow old and new
+ * Zserv headers to be distinguished from each other.
+ */
+#define ZEBRA_HEADER_MARKER              255
+
 /* Zebra route's types. */
 #define ZEBRA_ROUTE_SYSTEM               0
 #define ZEBRA_ROUTE_KERNEL               1
@@ -384,6 +434,18 @@ struct in_pktinfo
 #define ZEBRA_ROUTE_HSLS		 10
 #define ZEBRA_ROUTE_MAX                  11
 
+/* Note: whenever a new route-type or zserv-command is added the
+ * corresponding {command,route}_types[] table in lib/log.c MUST be
+ * updated! */
+
+/* Map a route type to a string.  For example, ZEBRA_ROUTE_RIPNG -> "ripng". */
+extern const char *zebra_route_string(unsigned int route_type);
+/* Map a route type to a char.  For example, ZEBRA_ROUTE_RIPNG -> 'R'. */
+extern char zebra_route_char(unsigned int route_type);
+/* Map a zserv command type to the same string, 
+ * e.g. ZEBRA_INTERFACE_ADD -> "ZEBRA_INTERFACE_ADD" */
+extern const char *zserv_command_string (unsigned int command);
+
 /* Zebra's family types. */
 #define ZEBRA_FAMILY_IPV4                1
 #define ZEBRA_FAMILY_IPV6                2
@@ -458,9 +520,9 @@ struct in_pktinfo
 typedef u_int16_t afi_t;
 typedef u_int8_t safi_t;
 
-/* Zebra types. */
+/* Zebra types. Used in Zserv message header. */
 typedef u_int16_t zebra_size_t;
-typedef u_int8_t zebra_command_t;
+typedef u_int16_t zebra_command_t;
 
 /* FIFO -- first in first out structure and macros.  */
 struct fifo
